{==============================================================================}
{
  Mail merge application: working with multiline field values

  How it works:
  - RVStyle1 has 4 predefined styles
    (1th text style is reserved for fields).
  - RichView1 is scanned for fields. When field is found, it is deleted,
    and its value is inserted in its place. Undocumented functions are used.
  Notes:
  - this demo uses field text as field codes. You can use tags.
  - this demo assigns TEXTSTYLE_NORMAL to the inserted text. You can use
    text style of the field instead.
}
{==============================================================================}

unit MainAppFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, RVItem,
  Dialogs, RVScroll, RichView, RVStyle, DB, DBTables, CRVFData, RVTable,
  StdCtrls, RVUni, RVTypes;

{$I RV_Defs.inc}

type
  TForm1 = class(TForm)
    RichView1: TRichView;
    Table1: TTable;
    RVStyle1: TRVStyle;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    function GetFieldValue(const Code: String): TStringList;
    procedure ReplaceFields(RVData: TCustomRVFormattedData);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

const
  TEXTSTYLE_NORMAL = 0;
  TEXTSTYLE_FIELD  = 1;
  TEXTSTYLE_HEADING1 = 2;
  TEXTSTYLE_HEADING2 = 3;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Table1.TableName := ExtractFilePath(Application.ExeName)+Table1.TableName;
  Table1.Open;
  // Creating document with fields
  RichView1.AddBreak;
  RichView1.AddNL('The Beatles', TEXTSTYLE_HEADING1, 1);
  RichView1.AddBreak;
  RichView1.AddNL('Yesterday:', TEXTSTYLE_HEADING2, 1);
  RichView1.AddNL('Yesterday', TEXTSTYLE_FIELD, 0);
  RichView1.AddBreak;
  RichView1.AddNL('Girl:', TEXTSTYLE_HEADING2, 1);
  RichView1.AddNL('Girl', TEXTSTYLE_FIELD, 0);
  RichView1.Format;
end;

// Converting text from String to internal representation
function ConvertStringToItemText(const Text: String;
  UnicodeItem: Boolean; CodePage: Cardinal): TRVRawByteString;
begin
  {$IFDEF RVUNICODESTR} // <-- declared in RV_Defs.inc
  if UnicodeItem then
    Result := RVU_GetRawUnicode(Text)
  else
    Result := TRVAnsiString(Text);
  {$ELSE}
  if UnicodeItem then
    Result := RVU_AnsiToUnicode(CodePage, Text)
  else
    Result := Text;
  {$ENDIF}
end;

procedure TForm1.ReplaceFields(RVData: TCustomRVFormattedData);
var i,j: Integer;
    Value: TStringList;
    s: String;
    sr: TRVRawByteString;
    ParaNo: Integer;
    BR, ContinuePara, Unicode: Boolean;
    CodePage: TRVCodePage;
    item: TRVTextItemInfo;
    table: TRVTableItemInfo;
    r,c: Integer;
begin
  for i := RVData.ItemCount-1 downto 0 do
    case RVData.GetItemStyle(i) of
      TEXTSTYLE_FIELD:
        begin
          // storing parameters of deleted items
          ParaNo := RVData.GetItemPara(i);
          BR     := RVData.GetItem(i).BR;
          ContinuePara := RVData.GetItem(i).SameAsPrev;
          // loading field value in the stream
          Value := GetFieldValue(RVData.GetItemText(i));
          if Value<>nil then begin
            // deleting the field code
            RVData.DeleteItems(i,1);
            // inserting the field value
            for j := Value.Count-1 downto 0 do begin
              item := RichViewTextItemClass.Create(RVData);
              s := Value[j];
              item.ParaNo := ParaNo;
              item.StyleNo := TEXTSTYLE_NORMAL;
              Unicode := RVData.GetRVStyle.TextStyles[TEXTSTYLE_NORMAL].Unicode;
              if Unicode then
                item.ItemOptions := item.ItemOptions + [rvioUnicode];
              CodePage := RVData.GetItemCodePage2(item);
              if j=0 then begin
                item.SameAsPrev := ContinuePara;
                item.BR         := BR;
               end;
              sr := ConvertStringToItemText(s, Unicode, CodePage);
              item.Inserting(RVData, sr, False);
              RVData.Items.InsertObject(i, sr, item);
              item.Inserted(RVData, i);
            end;
            Value.Free;
          end;
        end;
      rvsTable:
        begin
          // recursive call for table cells
          table := TRVTableItemInfo(RVData.GetItem(i));
          for r := 0 to table.Rows.Count-1 do
            for c := 0 to table.Rows[r].Count-1 do
              if table.Cells[r,c]<>nil then
                ReplaceFields(table.Cells[r,c]);
        end;
    end;
  RVData.Normalize;
end;


function TForm1.GetFieldValue(const Code: String): TStringList;
var s: String;
begin
  Result := nil;
  if not Table1.Locate('Code', Code, []) then
    exit;
  s :=  Table1.FieldByName('Memo').AsString;
  Result := TStringList.Create;
  Result.Text := s;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  ReplaceFields(RichView1.RVData);
  RichView1.Format;
end;

end.
